/***********************************************
 *  ~~ KBSliderX.js  V 4.3.4  ~~
 *
 *  Part of - KBSlider Project -
 *  Complete Div based /CSS/JS range slider
 *  (Not using HTML Input)
 *
 *  Copyright Kirk Siqveland 2022
 *
 *
 ***********************************************

    KBSlider provides Range-Slider function using only js, css and html
    without jQuery or any external PHP code.
    The demo CSS file does use PHP to simplify customization but the
    CSS code can be used from a simple CSS file if desired.

    This js file inserts the whole slider into your html file using
    an empty div with the class "KBSlider" and an id you use to 
    identify how it will be used, e.g.:

    <div class="KBSlider" id="volume" button="none"></div>

    Note: the "button" attribute flags whether to show progress on the
    slider button. The two options are "display" and "none"

    If there is a div with the class "feedback" we will use that to 
    display the progress in text as a Percent of progress along the slider.
    Code could be modified to use different increments.

 **/


 /*  TO USE : add code to your own JavaScript file, something like:

    document.addEventListener("DOMContentLoaded", function() {

       //Start KBSlider functions for Volume Slider:
          InitProgress("volume" )     ; // This inserts the Slider into the HTML based on a div with 
                                        // in this example:   <div class="KBSlider" id="volume" ></div>
          SetProgress( "volume", 80 ) ; // set the progress display to 80%
       setSliderDrag(  "volume" )     ; // activates the interactive functions (click, slide, drag)

    }) ;

*/


/*\ 
  EXTERNAL JS USAGE EXAMPLE:

     SetProgress(SliderID, pegProgress)                        // programatically Set Slider in Percent                     
     InitProgress(SliderID, minUnit=0, maxUnit=100, Step=1, BtnShift=2)     // Initialize Slider  Range
\*/  



  /*\*****   Template Call-Back: Insert your response to Changes from any Slider  ****\*****/
 /***\****        assign the slider and id in the html  e.g. id="volume";         *****\***/
/*****\***       Your primary js file should contain the Callback function        ******\*/

//     function KBSliderCallBack(ID, setProgress, setPixels = -1, maxPixels = -1){
//     // Add your Code to respond to slider Changes Here:
//         switch(ID){
//           case "volume":
//               audio.volume = setProgress/100 ;
//               break ;
//           case "songbar": 
//               audio.currentTime = setTime ; 
//               break ;
//         }
//     }    


var BtnShift   = -2 ; // adjust (usually negative number) to shift drag-button


/* ====================================================================== */
/*              G E N E R A L   P O S I T I O N I N G :                   */
/* ====================================================================== */

var DeBounce ;
var SliderSet = ["empty"]; // collection of all sliders in this document


/* ====================================================================== */
/*            B A S E   M O U S E   W H E E L   S T E P S                 */
/* ====================================================================== */
  var dyOpera =  40 ;       // Opera
  var dyFfox  =   3 ;       // Firefox;         
  var dyW3    = 120 ;       // IE/Safari/Chrome 
    if((navigator.userAgent.toLowerCase()).indexOf("mac os x")> 0){
        dyFfox  =   1 ;     // Firefox;        
        dyW3    =   3 ;     // IE/Safari/Chrome    
    }
 
/* ==================  set Position Slider by Percent  ================== */
    function SetProgress(ID, pegProgress){ 
        let  Slider  = "slider_" + ID  
        // console.log("SetProgress = "+Slider) ;
        window[Slider].Progress =  pegProgress                    ;
        window[Slider].ProgBar.style.width   =  pegProgress + "%" ;
        window[Slider].DragFrame.style.left  = (pegProgress - window[Slider].Offset) + "%" ;
        // window[Slider].title  = pegProgress + "%";
    }   

/* ==================  Position Slider by Pixels  ===================== */
    function setSlider(ID, pegWidth){    
      let Slider = "slider_"+ID                                 ;
        let testMax = window[Slider].maxVal                     ;
          if( pegWidth < 0 ){            pegWidth = 0           ;
          }else if(pegWidth > testMax){  pegWidth = testMax     ;}
        let pegProgress = Math.floor( pegWidth/testMax*100 )    ;

        window[Slider].Progress = pegProgress                     ;
        window[Slider].ProgBar.style.width   =  pegProgress + "%" ;
        window[Slider].DragFrame.style.left  = (pegProgress - window[Slider].Offset) + "%" ;                

      showString(ID, pegProgress)  ;
      KBSliderCallBack(ID, pegProgress) ;
    }

/* =========  Position Slider by Percent / or Tick Marks  ============== */
    function tapSlider(ID, pegShift){
      let Slider        = "slider_"+ID                          ;
        let testMax     = window[Slider].maxVal                   ;
        let pegProgress = window[Slider].Progress + pegShift      ;
          if(pegProgress < 0){          pegProgress =   0       ; 
          }else if(pegProgress > 100){  pegProgress = 100       ; }
  
        window[Slider].Progress = pegProgress                     ;
        window[Slider].ProgBar.style.width   =  pegProgress + "%" ;
        window[Slider].DragFrame.style.left  = (pegProgress - window[Slider].Offset) + "%" ;  

      showString(ID,pegProgress)                         ;
      KBSliderCallBack(ID, pegProgress) ;
    }    


/* =========  Position Slider with Mouse-Wheel Change  ==================== */
    function wheelMove(event){      
      let wheelid = document.activeElement.parentNode.id ;
      if (SliderSet.indexOf(wheelid) >= 0){

          if (!event) event = window.event     ;
            //flac needed to block page scroll
              event.preventDefault()           ;  
              event.stopPropagation()          ;
              event.stopImmediatePropagation() ;

        var delta=0, radDelta=event.wheelDelta, radDetail=event.detail ;
          if (radDetail){
            if (radDelta) return radDelta/dyOpera>0?1:-1 ;      // Opera
            else delta = -radDetail/dyFfox               ;      // Firefox;         
          } else delta =  radDelta/dyW3                  ;      // Chrome/IE/Safari        

          if(event.shiftKey){ 
                 delta = delta * 15 ; 
          }else{ delta = delta *  3 ; }

          tapSlider(wheelid, delta) ;

          //more flac to block page scroll
          event.returnValue = false;
          return false;
      } 
    }  

/* =============  Display Messages if desired  ================== */
    function showString(ID, msg, progSymbol = "%"){
      let thisSlider = document.getElementById(ID) ;
      thisSlider.querySelector(".fbDump ").innerHTML = msg+progSymbol ;
      thisSlider.querySelector(".dragbtn").innerHTML = msg+progSymbol ;
    }

/* ===========  Responsive / Dynamic Resize  ====================  */
    function resizeMe(){
        for (let tick = 1; tick < SliderSet.length; tick++){ 
          let Slider = "slider_"+SliderSet[tick];          
          window[Slider].setMaxVal();
        }
    }


/* ====================================================================== */
/*          P R O G R E S S   C H A N G E   M E T H O D S :               */
/* ====================================================================== */
// Called from js file controlling the slider for your app

function InitProgress(myID, minUnit=0, maxUnit=100, Step=1, BtnShift=2){
  /** e.g.   InitProgress("volume" ) ;     **/
    let Slider = "slider_" + myID ;
      console.log("InitSlider: "+Slider) ;

    SliderSet.push(myID) ;
    let thisSlider  = document.getElementById(myID) ;
    if( newSlider  = null){
      console.log ("failed to find slider "+myID) ;
      return ;
    }

    /**  INJECT the Slider Components   **/
      let addHTML   = '<div class="progbox" tabindex="0"><div class="progbar" >' ;
        addHTML     = addHTML+'<div id="mydragframe" class = "dragframe">'       ;
        addHTML     = addHTML+'<div id="mydragbtn"   class = "dragbtn">'         ;
        addHTML     = addHTML+'</div></div></div></div>'                         ;
        addHTML     = addHTML+'<div class="fbDump"></div>'                       ;
      thisSlider.innerHTML = addHTML ;

        let myprogbox = thisSlider.querySelector(".progbox")   ;
        let dragframe = thisSlider.querySelector(".dragframe") ;
        let dragbox   = dragframe.getBoundingClientRect()      ;        
        let dragbtn   = thisSlider.querySelector(".dragbtn")   ;
          if(thisSlider.getAttribute("button") !== ("display"||"Display"))
          {  dragbtn.classList.add("hidden")                   ; }        
        
        window[Slider] = { } ; //Declare a global variable collection

        // Assign values to the new collection
          window[Slider].ID        = myID ;
          window[Slider].ProgBox   = thisSlider.querySelector(".progbox")   ;
          window[Slider].ProgBar   = thisSlider.querySelector(".progbar")   ;
          window[Slider].DragFrame = thisSlider.querySelector(".dragframe") ; 
          window[Slider].Duration  =   0 ;
          window[Slider].Progress  =   0 ;
          window[Slider].minVal    =   0 ; 
          window[Slider].BtnShift  = BtnShift ;       
          window[Slider].minUnit   = minUnit  ;
          window[Slider].maxUnit   = maxUnit  ;  
          window[Slider].Step      = Step     ;
            window[Slider].setMaxVal = function () {
              let thisslider = document.getElementById(this.ID)                   ;
                this.maxVal  = (thisslider.querySelector(".progbox").getBoundingClientRect().width) ;
                this.Unit    = this.maxVal/(maxUnit-minUnit)                      ;
                this.Offset  = parseInt( ((dragbox.width/2)+BtnShift)/this.Unit ) ;
                this.ProgBar.style.width   =  this.Progress + "%" ;
                this.DragFrame.style.left  = (this.Progress - this.Offset) + "%" ;
            };
          window[Slider].setMaxVal() ;

        // console.log("InitProgress maxVal: "+window[Slider].maxVal+" - Unit: "+window[Slider].Unit+" - Offset: "+window[Slider].Offset ) ;

        /* ------ Click and Focus Event Listeners ----- */

        myprogbox.addEventListener("click", function(e){  
            let BCR  = this.getBoundingClientRect()      ;
            let peg  = (e.clientX - BCR.left)            ;
            let myID = this.parentNode.id ;
            this.style.opacity = "1"   ;
            this.focus();
            setSlider(myID, peg) ;
        });

        myprogbox.addEventListener('focus', function() {
            this.style.opacity = "1"   ;
        });
        myprogbox.addEventListener('focusout', function() {
            this.style.opacity = "0.8" ;
        });
        myprogbox.addEventListener('mouseover', function() {
            this.focus() ;
            // this.title = this.parentNode.id;
            let progBar = this.querySelector('.progbar');
            this.title  = parseInt((progBar.clientWidth / this.clientWidth||0)*100)+'%';
        });
        myprogbox.addEventListener('mouseout', function() {
            let Slider = "slider_" + this.parentNode.id ;
        });

    /* ==============  Keyboard Responses  ==================== */
        myprogbox.addEventListener('keydown', function(e) {
          /**    myID = this.parentNode.id ;        */
            // console.log("Keycode Down: " + e.keyCode) ;
            switch (e.keyCode) {
                // case 16: // Shift-Key
                //     shiftKey = true ;
                //     break ;
                case 35: // End-Key
                    tapSlider(myID, 100) ;
                    break ;
                case 36: // Home-Key
                    tapSlider(myID, 0) ;
                    break ;             
                case 37: // Arrow-Key Left
                    if(e.shiftKey){
                      tapSlider(myID, - 10) ;
                    }else{
                      tapSlider(myID, - 1) ;
                    }
                    break ;
                case 38: // Arrow-Key Up
                    if(e.shiftKey){
                      tapSlider(myID, - 10) ;
                    }else{
                      tapSlider(myID, - 1) ;
                    }
                    break ;                
                case 39: // Arrow-Key Right
                    if(e.shiftKey){
                      tapSlider(myID, + 10) ;
                    }else{
                      tapSlider(myID, + 1) ;
                    }
                    break ;
                case 40: // Arrow-Key Down
                    if(e.shiftKey){
                      tapSlider(myID, + 10) ;
                    }else{
                      tapSlider(myID, + 1) ;
                    }
                    break ;   
            }  /*/ -end-  Switch      /*/
        }); /*/ -end- Keydown     /*/
} /*/ -end- InitProgress   /*/


/* ====================================================================== */
/*          D R A G - T O U C H  M E T H O D S  :                         */
/* ====================================================================== */

/*    The Whole Drag/Touch-Process Bundled into One Function   */
    function setSliderDrag(sliderID) {
      let thisSlider = document.getElementById(sliderID)     ;             

      let progbox   = thisSlider.querySelector(".progbox")   ;
      let dragframe = thisSlider.querySelector(".dragframe") ;
      let currrect  = progbox.getBoundingClientRect()      ;
      let minUnit   = currrect.left                        ;
      let maxUnit   = currrect.left  + currrect.width      ;

      dragframe.onmousedown  = StartSliderDrag ;
      dragframe.ontouchstart = StartSliderPush ;       


    /* =========== Slider Mouse on Button Response ==================== */

      /**        Start a Drag sequence...        **/
        function StartSliderDrag(e) {
          // console.log("StartSliderDrag myID =  "+myID ) ;
            (e || window.event).preventDefault()                ;
            DeBounce = true ;

          /** set functions on MouseMove or MouseUp: */
            document.onmousemove  = sliderBtnDrag     ;
            document.onmouseup    = closeDragBtn      ;
        }

      // Show Drag Motion on document.mousemove
        function sliderBtnDrag(e) {   
          /** trim responses to parallel X to ProgBox */
          if(e.clientX>minUnit && e.clientX<maxUnit){ 
              (e || window.event).preventDefault()     ;
            /** calculate the new cursor position:    */
              let DragDelta = (e.clientX - minUnit)    ;
            /** set the element's new position:       */
              setSlider(sliderID, DragDelta )          ;
          }
        }

      // Finish a Drag Sequence (touch or mouse)
        function closeDragBtn() {
          /** stop moving when mouse button is released: */
              DeBounce = false            ;
              document.onmouseup   = null ;
              document.onmousemove = null ;
        }


    /* =========== Touch Slider Response ==================== */

        function StartSliderPush(e) {
          (e || window.event).preventDefault()    ;
            DeBounce = true ;
          /**  call function on Touch:  */
            document.ontouchmove  = sliderBtnPush ;        
            document.ontouchend   = EndSliderPush ;  
            document.ontouchcancel= EndSliderPush ;        
        }    
        function sliderBtnPush(e) {
            (e || window.event).preventDefault()  ;
          if(e.touches[0].clientX>minUnit&&e.touches[0].clientX<maxUnit){
            let DragDelta = (e.touches[0].clientX - minUnit) ;
            /** set the element's new position:       */
              setSlider("volume", DragDelta )     ;
          }
        }
        function EndSliderPush(e) {
            DeBounce = false             ;
            document.ontouchmove  = null ;        
            document.ontouchend   = null ;  
            document.ontouchcancel= null ;         
        }    
    } /*/ -end-  setSliderDrag() /*/



/* ================ Last Job : add Window Event Listeners =============== */

  window.onresize = resizeMe ;

  window.addEventListener('DOMMouseScroll', wheelMove, {passive: false}) ;  // Declared deprecated but 'wheel' does not work!
  window.addEventListener('mousewheel',    	wheelMove, {passive: false}) ;  // Declared deprecated but 'wheel' does not work!
  //window.addEventListener('wheel', wheelMove, {passive: false} ) ;


/* ====================================================================== */




